#include <stdio.h>
#include "loader.hpp"
#include "serial.hpp"

#include <iostream.h>

void swapBuffer(char * buf, int length)
{
	int i;
	for(i=length-2;i>=0;i--)
		buf[i+1]=buf[i];
}
void printBuffer(char * buf, int length)
{
	int i;
	cout << endl << "Buffer : ";
	for(i=0;i<length;i++)
		cout << (char)buf[i];
	cout << endl;
}


loaderCL::loaderCL()
{
	strcpy(flashFilename_, LOADER_DEFAULT_FILE_NAME);
	strcpy(bootFilename_,  LOADER_DEFAULT_BOOT_NAME);
	strcpy(bootFilename_,  LOADER_DEFAULT_PORT_NAME);
}

loaderCL::~loaderCL()
{
	serialCom_.close();
}

int loaderCL::setFlashFileName(char * filename)
{
	if(strlen(filename) >= LOADER_MAX_FILENAME_LENGTH)	
		return LOADER_ERROR;
	strcpy(flashFilename_, filename);
	return LOADER_SUCCESS;
}

int loaderCL::setBootFileName(char * filename)
{
	if(strlen(filename) >= LOADER_MAX_FILENAME_LENGTH)	
		return LOADER_ERROR;
	strcpy(bootFilename_, filename);
	return LOADER_SUCCESS;
}
int loaderCL::setSerialPort(char* portname)
{
	if(strlen(portname) >= LOADER_MAX_FILENAME_LENGTH)	
		return LOADER_ERROR;
	strcpy(portName_, portname);
	return LOADER_SUCCESS;
}
char * loaderCL::getBootFileName()
{
	return bootFilename_;
}

char * loaderCL::getFlashFileName()
{
	return flashFilename_;
}

int loaderCL::writeFlash(bool verbose)
{
	return writeFlash(getFlashFileName(), verbose);
}


int loaderCL::writeFlash(char * flashfilename, bool verbose)
{
	FILE * fptr;
	int c,retry;
	unsigned long l;
	char buffer[5];
	//////////////////////
	// download bootrom //
	//////////////////////
	if(verbose)
			cout << "Download Bootrom"<< endl;
	//** open serial port **//
	if(serialCom_.open(portName_,1200) != SERIAL_SUCCESS)
	{
		if(verbose)
			cout << "Serial COM init error"<<endl;
		return LOADER_COM_INIT_ERROR;
	}
	
	//** open bootfile **//
	fptr = fopen(getBootFileName(),"r");
	if(fptr == NULL)
	{
		serialCom_.close();
		if(verbose)
			cout << "Cannot open file " <<  getBootFileName() << endl;
		return LOADER_BOOT_FILE_OPEN_ERROR;
	}

	//** write bootefile **//
	c = fgetc(fptr);
	while(c != EOF)
	{
		buffer[0] = c;
		if(serialCom_.write(buffer,1) != SERIAL_SUCCESS)	// write a character on serial port
		{
			serialCom_.close();
			fclose(fptr);
			if(verbose)
				cout << "Boot rom download error"<<endl;
			return LOADER_BOOT_DOWNLOAD_ERROR;
		}
		c = fgetc(fptr);
	}	
	//** close bootfile **//
	fclose(fptr);
	//** close serial port **//
	serialCom_.close();


	//////////////////////
	// download flash   //
	//////////////////////
	if(verbose)
			cout << "Download Flash file"<< endl;
	//** open serial port **//
	if(serialCom_.open(portName_,9600) != SERIAL_SUCCESS)
	{
		if(verbose)
			cout << "Serial Com init error"<<endl;
		return LOADER_COM_INIT_ERROR;
	}
	//** open bootfile **//
	fptr = fopen(flashfilename,"r");
	if(fptr == NULL)
	{
		serialCom_.close();
		if(verbose)
			cout << "Cannot open file " <<  flashfilename << endl;
		return LOADER_FLASH_FILE_OPEN_ERROR;
	}
	//** send a special character for bootrom to save next data into ROM**//
	buffer[0] = 'I';
	if(serialCom_.write(buffer,1) != SERIAL_SUCCESS)	// write a character on serial port
	{
		serialCom_.close();
		fclose(fptr);
		if(verbose)
			cout << "Flash download writing error"<<endl;
		return LOADER_FLASH_DOWNLOAD_ERROR;
	}
	while(serialCom_.read(buffer,l) != SERIAL_SUCCESS 
			&& retry++ < LOADER_MAX_RETRY)// wait a character on serial port
		{	
			l=1;
		} 
		if( retry >= LOADER_MAX_RETRY)
		{
			serialCom_.close();
			fclose(fptr);
			if(verbose)
				cout << "Flash download writing error"<<endl;
			return LOADER_FLASH_DOWNLOAD_ERROR;
		}

	//** write bootefile **//
	c = fgetc(fptr);
	while(c != EOF)
	{
		swapBuffer(buffer,5);
		buffer[0] = c;
		if(serialCom_.write(buffer,1) != SERIAL_SUCCESS)	// write a character on serial port
		{
			serialCom_.close();
			fclose(fptr);
			//if(verbose) printBuffer(buffer,5);
			if(verbose)
				cout << "\nFlash download writing error"<<endl;
			return LOADER_FLASH_DOWNLOAD_ERROR;
		}
		retry = 0;
		l = 1;
		while(serialCom_.read(buffer,l) != SERIAL_SUCCESS 
			&& retry++ < LOADER_MAX_RETRY)// wait a character on serial port
		{	
			l=1;
		} 
		if( retry >= LOADER_MAX_RETRY)
		{
			serialCom_.close();
			fclose(fptr);
			//if(verbose) printBuffer(buffer,5);
			if(verbose)
				cout << "\nFlash download writing error"<<endl;
			return LOADER_FLASH_DOWNLOAD_ERROR;
		}
		if(verbose)
		{
			cout << buffer[0];
		}
		c = fgetc(fptr);
	}	
	if(verbose)
		cout << endl;
	//** close flashfile **//
	fclose(fptr);
	//** close serial port **//
	serialCom_.close();

	if(verbose)
		cout << "\nFlash download successfull"<<endl;
	return LOADER_SUCCESS;
}


